# wyvern-clangとの結合テスト

このディレクトリは主にwyvern-clangとWyvernの結合させるテストの置き場になっている。
zerooneと呼ばれるPUTを用いてテストを行っている。

### PUT: zeroone

zerooneのソースコードはこのディレクトリにあるzeroone.cpp。

PUTの内容は、変数をa, b, ..., hの8個用意し、8文字の01文字列を受け付け、

 - 1番目の文字が'1'ならa=1, 1番目の文字が'0'ならa=0
 - 2番目の文字が'1'ならb=1, 2番目の文字が'0'ならb=0

 :

 - 8番目の文字が'1'ならh=1, 8番目の文字が'0'ならh=0

をやるだけ。

PUTソースコード中では
```c
#define IFBRANCH(buf, idx, var) \
    if (buf[idx] == '1') { \
        var = 1; \
    } else { \
        var = 0; \
    }
```
というマクロを定義し、これを
```c
    IFBRANCH(buf, 0, a);
    IFBRANCH(buf, 1, b);
    IFBRANCH(buf, 2, c);
    IFBRANCH(buf, 3, d);
    IFBRANCH(buf, 4, e);
    IFBRANCH(buf, 5, f);
    IFBRANCH(buf, 6, g);
    IFBRANCH(buf, 7, h);
```
という形で8回使っている。
これは、bufの1文字目から8文字目までの比較によるif分岐を8つユニークに作りたいという意図で記述している。
これによって、入力00000000, ..., 11111111の256通りに対して、256通りの異なるカバレッジが記録されるはず。これをテストする。
わざわざマクロを定義して手で8回記述しているのは、最適化などでBasic Blockがまとめられないようにするため。
同様の意図で`-O0`オプションなども加えている。

### テストでやっていること

まず、入力00000000で通るcoverageを記録する。
次に、10000000, 010000000, ..., 00000001の8つの入力で通るcoverageを記録する。
PUTは上記のような挙動をしているので、
例えば00000000と10000000のcoverageの差分というのは、
「0番目の文字によって、通るか通らないかが変化する箇所」のはず。

より具体的には、上記のPUTのコードでいうところの`IFBRANCH(buf, 0, a);`で生成される、
if分岐中の`a = 0;`の部分と`a = 1;`の部分が、それらに相当する。

よって、これらを全部記録しておく。

あとは、入力00000000 〜 11111111の256通りに対して、
記録したことをもとに正しいcoverageであるかどうかを判定する。

すべての入力のcoverageは、記録したものの"組み合わせ"によって表現できるはず。
たとえば、入力01010000のcoverageというのは、「すべての入力が通るBasic Block」に、
「入力01000000が通って、入力00000000が通らないBasic Block」と
「入力00010000が通って、入力00000000が通らないBasic Block」を
足し合わせたものになっていると考えられる。

逆に、確実に通らないBasic Blockも存在している。
例えば、「入力10000000が通って、入力00000000が通らないBasic Block」は、入力01010000のcoverageにおいて通っているはずがないし（1文字目が0なので）、
「入力00000000が通って、入力01000000が通らないBasic Block」も通るはずがない
（「入力01000000が通って、入力00000000が通らないBasic Block」を通っているため）。

これらについてすべてassertを掛けているのがテストコードの内容。
`struct DiffInfo`内に、前述の記録を追加していく。
メンバpassedには例えば「入力01000000が通って、入力00000000が通らないBasic Block」が入る。
メンバignoredには例えば「入力00000000が通って、入力01000000が通らないBasic Block」が入る。
これを10000000, 01000000, ..., 00000001の8つに対してやる。

あとは、passedとignoredを元に、256通りの入力に対してcoverageが通るべきBasic Blockを通っており、通らないべきBasic Blockを通っていないことを確認している。 

### テストが通らなかったとき

AFL covの方のテストのみが失敗するときは、まずAFLが各Basic Block（ひいてはedge）に割り当てるハッシュが衝突していないかを気にしてほしい。
